#!/usr/bin/python
 
import cv2
import numpy as np
import random
 
# Check if a point is inside a rectangle
def rect_contains(rect, point) :
    if point[0] < rect[0] :
        return False
    elif point[1] < rect[1] :
        return False
    elif point[0] > rect[2] :
        return False
    elif point[1] > rect[3] :
        return False
    return True
 
# Draw a point
def draw_point(img, p, color ) :
    cv2.circle (img, p, 3, color, cv2.cv.CV_FILLED, cv2.CV_AA, 0)
 
 
# Draw delaunay triangles
def draw_delaunay(img, subdiv, delaunay_color ) :
 
    triangleList = subdiv.getTriangleList();
    size = img.shape
    r = (0, 0, size[1], size[0])
 
    for t in triangleList :
        pt1 = (t[0], t[1])
        pt2 = (t[2], t[3])
        pt3 = (t[4], t[5])
        if rect_contains(r, pt1) and rect_contains(r, pt2) and rect_contains(r, pt3) :
            cv2.line(img, pt1, pt2, delaunay_color, 1, cv2.CV_AA, 0)
            cv2.line(img, pt2, pt3, delaunay_color, 1, cv2.CV_AA, 0)
            cv2.line(img, pt3, pt1, delaunay_color, 1, cv2.CV_AA, 0)
 
if __name__ == '__main__':
 
    # Define window names
    win_delaunay = "Delaunay Triangulation"
 
    # Turn on animation while drawing triangles
    animate = True
     
    # Define colors for drawing.
    delaunay_color = (255,0,0)
    points_color = (0, 0, 255)
 
    # Read in the image.
    img = cv2.imread("rob.png");
     
    # Keep a copy around
    img_orig = img.copy();
     
    # Rectangle to be used with Subdiv2D
    size = img.shape
    rect = (0, 0, size[1], size[0])
     
    # Create an instance of Subdiv2D
    subdiv = cv2.Subdiv2D(rect);
 
    # Create an array of points.
    points = [];
    points.append((124, 594));
    points.append((121, 677));
    points.append((127, 756));
    points.append((138, 832));
    points.append((208, 951));
    points.append((312, 1032));
    points.append((394, 1048));
    points.append((475, 1031));
    points.append((567, 950));
    points.append((624, 847));
    points.append((636, 772));
    points.append((639, 685));
    points.append((633, 593));
    points.append((509, 350));
    points.append((389, 325));
    points.append((267, 353));
    points.append((275, 509));
    points.append((221, 506));
    points.append((174, 540));
    points.append((222, 540));
    points.append((271, 543));
    points.append((324, 542));
    points.append((435, 539));
    points.append((488, 504));
    points.append((546, 503));
    points.append((591, 543));
    points.append((543, 538));
    points.append((492, 540));
    points.append((498, 566));
    points.append((258, 568));
    points.append((300, 601));
    points.append((278, 587));
    points.append((256, 580));
    points.append((235, 585));
    points.append((214, 598));
    points.append((235, 608));
    points.append((256, 613));
    points.append((278, 609));
    points.append((256, 596));
    points.append((498, 593));
    points.append((455, 599));
    points.append((477, 583));
    points.append((498, 577));
    points.append((520, 583));
    points.append((541, 595));
    points.append((520, 605));
    points.append((499, 609));
    points.append((477, 606));
    points.append((423, 678));
    points.append((381, 679));
    points.append((339, 681));
    points.append((333, 758));
    points.append((384, 740));
    points.append((435, 754));
    points.append((470, 715));
    points.append((460, 753));
    points.append((386, 779));
    points.append((307, 758));
    points.append((294, 718));
    points.append((263, 834));
    points.append((316, 820));
    points.append((361, 818));
    points.append((388, 824));
    points.append((413, 816));
    points.append((458, 816));
    points.append((511, 825));
    points.append((444, 835));
    points.append((388, 843));
    points.append((330, 840));
    points.append((329, 845));
    points.append((388, 851));
    points.append((444, 842));
    points.append((476, 853));
    points.append((434, 868));
    points.append((389, 873));
    points.append((342, 872));
    points.append((300, 860));
 
    # Insert points into subdiv
    for p in points :
        subdiv.insert(p)
         
    # Draw delaunay triangles
    draw_delaunay (img, subdiv, (255, 0, 0));
 
    # Draw points
    for p in points :
        draw_point(img, p, (0,0,255))

    cv2.imwrite("OUTPUT-rob.png", img);
